/* Define to 1 if you have the declaration of `mpz_powm', and to 0 if you
   don't. */
#undef HAVE_DECL_MPZ_POWM

/* Define to 1 if you have the declaration of `mpz_powm_sec', and to 0 if you
   don't. */
#undef HAVE_DECL_MPZ_POWM_SEC

/* Define to 1 if you have the `gmp' library (-lgmp). */
#undef HAVE_LIBGMP

/* Define to 1 if you have the `mpir' library (-lmpir). */
#undef HAVE_LIBMPIR

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1
